//------------------------------------------------------------------------------
//  TextEdit.inc       by Andreas Dreier
//                        Dorfleite 17 - 86641 Staudheim - Germany
//                        mailto: andreas@dreier-privat.de
//
//   This include shows a complex usage of RIC files for a text intput on the
//   NXT only by using of 3 buttons (left, center and right). The input will be
//   cascaded in two levels:
//
//     1st level: Select a group where the letter is included
//     2nd level: Select the letter inside the group - or go back to 1set level
//
//    1st level            2nd level
//    ----------------     ---------------------------------------------
//
//                                +------------------------------------+
//                         +----->| Group 1:  A B C D E F G H I J K L  |
//                         |      +------------------------------------+
//                         |
//                         |      +------------------------------------+
//                         +----->| Group 2:  M N O P Q R S T U V W X  |
//                         |      +------------------------------------+
//                         |
//                         |      +------------------------------------+
//                         +----->| Group 2:  Y Z 0 1 2 3 4 5 6 7 8 9  |
//     +--------------+    |      +------------------------------------+
//     | Select group |----+
//     +--------------+    |      +------------------------------------+
//                         +----->| Group 2:  a b c d e f g h i j k l  |
//                         |      +------------------------------------+
//                         |
//                         |      +------------------------------------+
//                         +----->| Group 1:  m n o p q r s t u v w x  |
//                         |      +------------------------------------+
//                         |
//                         |      +------------------------------------+
//                         +----->| Group 2:  y z   @ + - * / " . : ;  |
//                                +------------------------------------+
//
//   In the first line of the screen you can find the text which was entered.
//
//   Additional files on NXT:  TextEdit.ric
//------------------------------------------------------------------------------



#define   STAY_GROUP   FALSE
#define   LEAVE_GROUP  TRUE


//------------------------------------------------------------------------------
//  Routine displayed the current text on the top line and shows the menue for
//  selecting the group or a letter. Function can only be left by pressing the
//  center button. Left and right will only change the selected group/letter.
//  At the end the function returns the last selected position.
//------------------------------------------------------------------------------

int ButtonInput( string text,            // text for displaying on top line
                 int    menuelevel,      // number of menue level
                 int    selection,       // index of start selection
                 int    max_selection )  // max value for selection
{
    int te[1];
    int key;
    bool btn_pressed;
    bool selection_done;

    selection_done = FALSE;
    while( ! selection_done )
    {
        // Show already entered text in the title - if it is too long than show
        // only the last 16 characters.
        
        if( StrLen( text ) <= 16 )
            TextOut( 0,LCD_LINE1,text,TRUE );
        else
            TextOut( 0,LCD_LINE1,SubStr( text, StrLen( text )-16,16 ) );

        // Show the right menue and selection of the RIC file
            
        te[0] = menuelevel*20+selection;
        GraphicOutEx( 0,0, "TextEdit.ric", te );
        
        Wait( 50 );

        btn_pressed = FALSE;
        while( ! btn_pressed )
        {
            // Wehen left button will be pressed then decrement selection.
            // When selection is less than 0 than set selection to max_selection
        
            if( ButtonPressed( BTNLEFT, TRUE ) )
            {
                selection--;
                if( selection < 0 )
                    selection = max_selection;
                btn_pressed = TRUE;
                
                // Wait until butten will be released
                while( ButtonPressed( BTNLEFT, TRUE ) )
                       ;
            }
            
            // When the right button will be pressed than increment selection.
            // When selection is too high than set it back to 0
            
            if( ButtonPressed( BTNRIGHT, TRUE ) )
            {
                selection++;
                if( selection > max_selection )
                    selection = 0;
                btn_pressed = TRUE;
                
                // Wait until butten will be released
                while( ButtonPressed( BTNRIGHT, TRUE ) )
                       ;
            }
            
            // When the center butten will be pressed then leave the function
            
            if( ButtonPressed( BTNCENTER, TRUE ) )
            {
                selection_done = TRUE;
                btn_pressed    = TRUE;

                // Wait until butten will be released
                while( ButtonPressed( BTNCENTER, TRUE ) )
                       ;
            }
        }
    }
    
    // Return the position of the current selection for further usage
    
    return( selection );
}

//------------------------------------------------------------------------------
//  Select a letter from the given group and add it to the end of the text.
//  At the end return the new string.
//  With parameter jump_back you can decide to leave the letter menue when a
//  letter was pressed - or stay in the letter menue for entering another letter
//     jump_back = TRUE  --> Jump back when letter was selected
//     jump_back = FALSE --> Stay in the letter menue
//  The parameter letters includes all letters of the current letter menue.
//------------------------------------------------------------------------------

string TextSubEdit( string text,          // text which was already input
                    int    basismenue,    // nr of character group
                    bool   jump_back,     // jump back after letter selection
                    string letters     )  // letters of actual group
{
     int    selection;
     string letter;

     selection = 6;
     do
     {
        selection = ButtonInput( text, basismenue, selection, 12 );
        
        if( selection != 6 )
        {
            letter = SubStr( letters, selection,1 );
            text   = StrCat( text, letter );
        }
            
     }
     while ( ( selection != 6 ) &
             ( ! jump_back    )   );
     
     return( text );
}

//------------------------------------------------------------------------------
//  This is the top level menue for selecting the letter group. If the string
//  parameter text is set than you can modify the current string.
//------------------------------------------------------------------------------

string TextEdit( string text,            // text which should be modified
                 bool   jump_back )      // jump back after letter selection
{
     int menue;
     int key_input;
     
     menue = 4;
     do
     {
        menue = ButtonInput( text, 0, menue, 7 );
        
        switch( menue )
        {
           case 0 : text = TextSubEdit( text, 1, jump_back, "ABCDEF~GHIJKL" );
                    break;

           case 1 : text = TextSubEdit( text, 2, jump_back, "MNOPQR~STUVWX" );
                    break;

           case 2 : text = TextSubEdit( text, 3, jump_back, "YZ0123~456789" );
                    break;
                    
           case 3 : text = SubStr( text, 0, StrLen( text )-1 );
                    break;
                    
           case 4 : // exit
                    break;
                    
           case 5 : text = TextSubEdit( text, 4, jump_back, "abcdef~ghijkl" );
                    break;

           case 6 : text = TextSubEdit( text, 5, jump_back, "mnopqr~stuvwx" );
                    break;

           case 7 : text = TextSubEdit( text, 6, jump_back, "yz @+-~*/\".:;" );
                    break;
        }
     }
     while( menue != 4 );
     
     return( text );
}

//------------------------------------------------------------------------------
//  TextEdit.inc - End of file
//------------------------------------------------------------------------------
